#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (C) Geoffrey T. Dairiki
""" A tool to convert wheels to eggs.
"""
from __future__ import absolute_import

from collections import defaultdict
import email
from itertools import chain
import logging
import os
import posixpath
import py_compile
import shutil
import sys
import tempfile
from textwrap import dedent
from zipfile import ZipFile, ZIP_DEFLATED

import click
from distlib.markers import interpret
import distlib.scripts
from distlib.util import get_export_entry
from distlib.wheel import Wheel
import pkg_resources
from six import binary_type, text_type, PY3

log = logging.getLogger(__name__)

try:
    from importlib.util import cache_from_source
except ImportError:             # python < 3.4
    try:
        from imp import cache_from_source
    except ImportError:         # python < 3.2
        def cache_from_source(path):
            root, ext = os.path.splitext(path)
            return root + '.pyc'

try:
    import sysconfig
except ImportError:            # pragma: NO COVER
    # python < 2.7
    # FIXME: incorrect on non-posix systems
    EXT_SUFFIX = '.so'
else:
    EXT_SUFFIX = (sysconfig.get_config_var('EXT_SUFFIX')
                  or sysconfig.get_config_var('SO'))


class file_cm(object):
    """ Add context manager methods to dumb file-like instances.

    """
    def __init__(self, fp):
        self._fp = fp

    def __enter__(self):
        return self._fp

    def __exit__(self, typ, inst, tb):
        self._fp.close()

    def __getattr__(self, name):
        return getattr(self._fp, name)


def unsplit_sections(sections):
    """ This is essentially the inverse of pkg_resources.split_sections.
    """
    lines = []
    for section, content in sections:
        if section is not None:
            lines.append("[%s]" % section)
        lines.extend(content)
    return join_lines(lines)


def join_lines(lines):
    return '\n'.join(list(lines) + ['']).encode('utf-8')


def bytes_(s, encoding='latin1', errors='strict'):
    if not isinstance(s, binary_type):
        s = s.encode(encoding, errors)
    return s


def _get_requires_json(wheel_metadata):
    """ Compute requirements, grouped by extra.

    This expects wheel_metadata.run_requires to be in the JSON format
    as described at
    https://www.python.org/dev/peps/pep-0426/#dependencies

    """
    by_extra = defaultdict(set)
    for req in wheel_metadata.run_requires:
        extra = req.get('extra')
        marker = req.get('environment')
        if not marker or interpret(marker):
            by_extra[extra].update(req['requires'])

    for extra in sorted(by_extra.keys(),
                        key=lambda extra: (extra is not None, extra)):
        yield extra, sorted(by_extra[extra])


def _get_requires_rfc822(wheel_metadata):
    """ Compute requirements, grouped by extra.

    This expects wheel_metadata.run_requires to be a list of strings,
    as when metadata comes from legacy RFC822 formatted metadata.

    """
    run_requires = wheel_metadata.run_requires

    def get_reqs(extra=None):
        for req in run_requires:
            assert isinstance(req, text_type)
            req_, sep, marker = req.rpartition(';')
            if not sep:
                yield req
            elif interpret(marker.lstrip(), {'extra': extra}):
                yield req_.rstrip()

    reqs = list(get_reqs())
    if reqs:
        yield None, reqs

    unconditional = set(reqs)

    def is_conditional(req):
        return req not in unconditional

    for extra in wheel_metadata.extras:
        reqs = list(filter(is_conditional, get_reqs(extra)))
        if reqs:
            yield extra, reqs


class EggInfoBase(object):
    """ Egg metadata.

    An instance of this class is an iterable of ``filename, content``
    pairs, where each pair represents a metadata file which should be
    written to the packages ``EGG-INFO`` directory.

    """
    def __init__(self, wheel_metadata, installed_files, metadata_files,
                 zip_safe=False):
        self.wheel_metadata = wheel_metadata
        self.installed_files = installed_files
        self.metadata_files = metadata_files
        self.zip_safe = zip_safe

    def files(self):
        # XXX: dependency_links.txt?

        yield 'PKG-INFO', join_lines(self.pkg_info)

        for name in ('requires', 'entry_points'):
            sections = getattr(self, name)
            if sections:
                yield "%s.txt" % name, unsplit_sections(sections)

        for name in ('top_level', 'namespace_packages',
                     'native_libs', 'eager_resources'):
            lines = getattr(self, name)
            if lines:
                yield "%s.txt" % name, join_lines(lines)

        if self.zip_safe:
            yield 'zip-safe', b''
        else:
            yield 'not-zip-safe', b''

    __iter__ = files

    @property
    def pkg_info(self):
        index_data = self._get_index_data()

        yield "Metadata-Version: 1.1"

        for name in ("Name", "Version", "Summary", "Home-page",
                     "Author", "Author-email", "License"):
            key = name.lower().replace('-', '_')
            yield "%s: %s" % (name, index_data.get(key, 'UNKNOWN'))

        if 'download_url' in index_data:
            yield "Download-URL: %s" % index_data['download_url']

        description = index_data.get('description', 'UNKNOWN').split('\n')
        yield "Description: %s" % description[0]
        for continuation in description[1:]:
            yield "        %s" % continuation

        if 'keywords' in index_data:
            yield "Keywords: %s" % ','.join(index_data['keywords'])

        for platform in index_data.get('platform', ['UNKNOWN']):
            yield "Platform: %s" % platform

        if 'classifiers' in index_data:
            for classifier in index_data['classifiers']:
                yield "Classifier: %s" % classifier

    def _get_index_data(self):
        return self.wheel_metadata.todict()

    @property
    def entry_points(self):     # pragma: NO COVER
        raise NotImplementedError()

    @property
    def namespace_packages(self):  # pragma: NO COVER
        raise NotImplementedError()

    @property
    def top_level(self):
        top_level = set()
        for path in self.installed_files:
            lpath = path.lower()
            for suffix in ('.py', EXT_SUFFIX):
                if lpath.endswith(suffix):
                    root = path[:-len(suffix)]
                    break
            else:
                continue
            top, sep, tail = root.partition('/')
            top_level.add(top)
        return sorted(top_level)

    @property
    def native_libs(self):
        # wheel (at least as of 0.24.0) does not generate a native_libs.txt
        def is_ext_mod(fn):
            root, ext = posixpath.splitext(fn)
            return ext.lower() in ('.so', '.dll', '.dylib')
        return sorted(set(filter(is_ext_mod, self.installed_files)))

    def eager_resources(self):  # pragma: NO COVER
        raise NotImplementedError()

    def requires(self):         # pragma: NO COVER
        raise NotImplementedError()


class EggInfo_Legacy(EggInfoBase):
    """Compute egg metadata from PEP427 Wheel 1.0 metadata.

    This is for ``.whl`` produced by the current version (0.24.0) of
    ``wheel``\s ``bdist_wheel`` setup command.  It uses the
    ``METADATA`` which purports to be of metadata version 2.0, but is
    in the RFC822 format, rather than JSON as specified by :pep:`426`.

    """
    def _get_index_data(self):
        index_data = super(EggInfo_Legacy, self)._get_index_data()
        description = index_data.get('description')
        if description is None:
            # distlib (as of 0.2.1) does not manage to parse the
            # description from the METADATA files written by wheel's
            # bdist_wheel.
            description = self._get_description()
        else:
            # Setuptools writes the description with continuation
            # lines indented eight spaces.  Distlib expects
            # continuation lines to be prefixed by seven spaces and a
            # pipe (per PEP 345.)  As a result distlib does not seem
            # to remove setuptools' eight space prefixes :-/
            description = '\n'.join(
                line[8:] if line.startswith(' ' * 8) else line
                for line in description.strip().splitlines()
                )
        index_data['description'] = description
        return index_data

    def _get_description(self):
        log.debug("Reading description from METADATA")
        metadata = self.metadata_files['METADATA']
        # NB: Under py3k, message_from_file seems to want a str, but
        # with bytes in it (not characters.)
        if PY3:                 # pragma: NO COVER
            metadata = metadata.decode('latin1')
        msg = email.message_from_string(metadata)
        body = msg.get_payload(decode=True)
        return bytes_(body).decode('utf-8')

    @property
    def entry_points(self):
        sections = pkg_resources.split_sections(
            self._read_metadata('entry_points.txt'))
        return [(section, lines) for section, lines in sections if lines]

    @property
    def namespace_packages(self):
        return list(self._read_metadata('namespace_packages.txt'))

    @property
    def top_level(self):
        # FIXME: maybe depend on wheel version?
        if self._metadata_exists('top_level.txt'):
            return list(self._read_metadata('top_level.txt'))
        else:
            # wheel < 0.10 does not write a top_level.txt
            return super(EggInfo_Legacy, self).top_level

    @property
    def eager_resources(self):
        return list(self._read_metadata('eager_resources.txt'))

    @property
    def requires(self):
        wheel_metadata = self.wheel_metadata
        is_legacy = wheel_metadata._legacy is not None
        if is_legacy:
            # With older versions of distlib, when reading wheels
            # built with older versions of wheel,
            # metadata.run_requires is a list of strings in the RFC822
            # metadata style.
            # (E.g. distlib==0.2.4, wheel==0.25.0)
            get_requires = _get_requires_rfc822
        else:
            # With recent versions of distlib, or with wheels built
            # by recent versions of wheel, run_requires is a list of dicts
            # in the "JSON" format described by PEP-426.
            get_requires = _get_requires_json

        return list(get_requires(wheel_metadata))

    def _read_metadata(self, name):
        """ Read a .txt format metadata file from .whl file.
        """
        content = self.metadata_files.get(name, b'')
        content = content.decode('utf-8')
        return pkg_resources.yield_lines(content)

    def _metadata_exists(self, name):
        return name in self.metadata_files


class EggInfo(EggInfoBase):
    """ Compute egg metadata from PEP491 Wheel 1.1? metadata.

    This uses the JSON metadata version 2.0 as specified in :pep:`426`.

    **NB**: Currently, I know of know packages which produce wheels of
    this type, so this code is untested.

    """
    @property
    def entry_points(self):
        return [
            (section, ["%s = %s" % item for item in entries.items()])
            for section, entries in self.wheel_metadata.exports.items()
            ]

    @property
    def namespace_packages(self):
        return self.wheel_metadata.namespaces

    @property
    def eager_resources(self):
        # FIXME: not sure what to do.
        return []

    @property
    def requires(self):
        return list(_get_requires_json(self.wheel_metadata))


def list_installed_files(wheel):
    wheel_file = os.path.join(wheel.dirname, wheel.filename)
    info_pfx = "{0.name}-{0.version}.dist-info/".format(wheel)
    data_pfx = "{0.name}-{0.version}.data/".format(wheel)
    purelib_pfx = "{0.name}-{0.version}.data/purelib/".format(wheel)
    platlib_pfx = "{0.name}-{0.version}.data/platlib/".format(wheel)

    installed = set()
    with file_cm(ZipFile(wheel_file, 'r')) as zf:
        for path in zf.namelist():
            if path.endswith('/'):
                # directory
                continue        # pragma: NO COVER
            elif path.startswith(info_pfx):
                continue
            elif path.startswith(purelib_pfx):
                installed_path = path[len(purelib_pfx):]
            elif path.startswith(platlib_pfx):
                installed_path = path[len(platlib_pfx):]
            elif path.startswith(data_pfx):
                continue
            else:
                installed_path = path
            installed.add(installed_path)
    return installed


def read_metadata_files(wheel):
    wheel_file = os.path.join(wheel.dirname, wheel.filename)
    info_pfx = "{0.name}-{0.version}.dist-info/".format(wheel)
    metadata_files = {}
    with file_cm(ZipFile(wheel_file, 'r')) as zf:
        for path in zf.namelist():
            if path.startswith(info_pfx) and not path.endswith('/'):
                with file_cm(zf.open(path)) as fp:
                    name = path[len(info_pfx):]
                    metadata_files[name] = fp.read()
    return metadata_files


def is_zip_safe(wheel):
    # XXX: Py3k seems not to be able to load .pyc from zipped eggs.
    # More specifically, it will load .pyc files which are placed next
    # to their .py files (old-style), however compiled byte-code in
    # :pep:`3147` style *repository directories* (__pycache__) seem
    # not be loaded from a zip file.  OTOH, zipped eggs generated by
    # setuptools bdist_egg command suffer the same issue.  Punt for
    # now...

    # FIXME: currently distlib's WHEEL.is_mountable always
    # returns true
    #
    # Note that currently this function is unused.
    return wheel.is_mountable()


def get_wheel_version(wheel):
    return tuple(map(int, wheel.info['Wheel-Version'].split('.')))


def egg_metadata(wheel, egg_metadata_class=None):
    if get_wheel_version(wheel) < (1, 1):
        egg_metadata_class = EggInfo_Legacy
    else:
        egg_metadata_class = EggInfo

    return egg_metadata_class(
        wheel.metadata,
        installed_files=list_installed_files(wheel),
        metadata_files=read_metadata_files(wheel),
        zip_safe=False)


class StubLoaders(object):
    """ Generate stub loaders.

    Generate stub loaders for namespace packages and extension modules.

    An instance of this class is an iterable of ``filename, content``
    pairs, where each pair represents a stub-loader file which should be
    written to the packages egg.

    """
    NAMESPACE_STUB = dedent("""
        try:
            __import__('pkg_resources').declare_namespace(__name__)
        except ImportError:
            __path__ = __import__('pkgutil').extend_path(__path__, __name__)
        """).lstrip()

    EXT_STUB_TEMPLATE = dedent("""
        def __bootstrap__():
            global __bootstrap__, __loader__, __file__
            import sys, pkg_resources, imp
            __file__ = pkg_resources.resource_filename(__name__, {extname!r})
            __loader__ = None; del __bootstrap__, __loader__
            imp.load_dynamic(__name__,__file__)
        __bootstrap__()
        """).lstrip()

    def __init__(self, egg_info, egg_name=''):
        self.egg_info = egg_info
        self.egg_name = egg_name

    def __iter__(self):
        # XXX: don't need namespace stubs for py3k, if egg is unpacked,
        # however they are still necessary if running from zipped egg.
        # For now, always include them.
        stubs = self.namespace_stubs()
        if self.egg_info.zip_safe:
            stubs = chain(stubs, self.extension_stub_loaders())

        for arcname, content in stubs:
            yield arcname, content
            yield self.byte_compile(arcname, content)

    def namespace_stubs(self):
        """ Create __init__.py for namespace packages
        """
        content = self.NAMESPACE_STUB.encode('utf-8')
        for package in self.egg_info.namespace_packages:
            parts = package.split('.') + ['__init__.py']
            arcname = posixpath.join(*parts)
            yield arcname, content

    def extension_stub_loaders(self):
        """ Stub loaders for extension modules.

        Only needed (I think) if zip_safe is set.

        """
        for extmod in self.egg_info.native_libs:
            if extmod.endswith(EXT_SUFFIX):
                head, extname = posixpath.split(extmod)
                stubname = extmod[:-len(EXT_SUFFIX)] + '.py'
                stub_loader = self.EXT_STUB_TEMPLATE.format(extname=extname)
                content = stub_loader.encode('utf-8')
                yield stubname, content

    def byte_compile(self, arcname, content):
        py_path = os.path.join(*arcname.split('/'))
        pyc_path = cache_from_source(py_path)
        arcname_pyc = '/'.join(pyc_path.split(os.path.sep))

        diagnostic_name = posixpath.join(self.egg_name, arcname)
        with tempfile.NamedTemporaryFile() as dst:
            with tempfile.NamedTemporaryFile() as src:
                src.write(content)
                src.flush()
                py_compile.compile(src.name, dst.name, diagnostic_name, True)
            return arcname_pyc, dst.read()


class EggWriter(object):
    def __init__(self, wheel_file):
        wheel = Wheel(wheel_file)

        if not wheel.is_compatible():
            # Workaround for https://bitbucket.org/pypa/distlib/issues/93
            log.warning(
                "WARNING: Distlib reports that %s is not compatible "
                "with this platform. "
                "(Note that on some versions of python, "
                "distlib's detection of compatible ABIs is broken. "
                "See distlib issue #93.)",
                wheel_file)
        wheel.verify()

        self.wheel = wheel

    def build_egg(self, destdir):
        wheel = self.wheel
        outfile = os.path.join(destdir, self.egg_name)
        egg_info = egg_metadata(wheel)
        log.warning("Converting %s to %s", wheel.filename, outfile)

        if not os.path.isdir(destdir):
            log.info("Creating dist directory %s", destdir)
            os.makedirs(destdir)

        with file_cm(ZipFile(outfile, 'w', ZIP_DEFLATED)) as zf:
            builddir = tempfile.mkdtemp()
            try:
                for arcname, filename in self.unpack_wheel(builddir):
                    zf.write(filename, arcname)
            finally:
                shutil.rmtree(builddir)

            stub_loaders = StubLoaders(egg_info, self.egg_name)
            for arcname, content in stub_loaders:
                zf.writestr(arcname, content)

            for filename, content in egg_info:
                arcname = 'EGG-INFO/%s' % filename
                zf.writestr(arcname, content)

        return outfile

    def unpack_wheel(self, libdir):
        wheel = self.wheel
        name_version = '%s-%s' % (wheel.name, wheel.version)
        data_dir = os.path.join(libdir, '%s.data' % name_version)
        egginfo_dir = os.path.join(libdir, 'EGG-INFO')
        paths = {
            'purelib': libdir,
            'platlib': libdir,
            'scripts': os.path.join(egginfo_dir, 'scripts'),
            }
        for where in 'prefix', 'headers', 'data':
            paths[where] = os.path.join(data_dir, where)

        for path in paths.values():
            if not os.path.isdir(path):
                log.debug("Creating directory %s", path)
                os.makedirs(path)

        maker = ScriptCopyer(None, None)
        wheel.install(paths, maker, warner=warner)

        subdirs = [()]
        while subdirs:
            path = subdirs.pop(0)
            for fn in os.listdir(os.path.join(libdir, *path)):
                topdir = len(path) == 0
                fpath = path + (fn,)
                filepath = os.path.join(libdir, *fpath)
                if os.path.isdir(filepath):
                    if topdir and fn.lower().endswith('.dist-info'):
                        # Omit .dist-info directory
                        continue
                    subdirs.append(fpath)
                else:
                    if topdir and fn.endswith('-nspkg.pth'):
                        # Omit *-nspkg.pth file
                        continue
                    arcname = posixpath.join(*fpath)
                    yield arcname, filepath

    @property
    def egg_name(self):
        wheel = self.wheel
        name = pkg_resources.safe_name(wheel.name)
        version = pkg_resources.safe_version(wheel.version)
        pyver = 'py%d.%d' % sys.version_info[:2]
        bits = [pkg_resources.to_filename(name),
                pkg_resources.to_filename(version),
                pyver]
        if any(abi != 'none' or arch != 'any'
               for pyver, abi, arch in wheel.tags):
            # not pure python
            bits.append(pkg_resources.get_build_platform())
        return '-'.join(bits) + '.egg'


def warner(software_wheel_version, file_wheel_version):
    log.info("Wheel version mismatch: software=%r, file=%r",
             software_wheel_version, file_wheel_version)


class ScriptCopyer(distlib.scripts.ScriptMaker):
    """ A ScriptMaker which does not create script wrappers.

    It does copy the scripts from the source.
    """
    def make(self, specification, options=None):
        if get_export_entry(specification):
            log.debug("Not building script: %s", specification)
            return []
        return super(ScriptCopyer, self).make(specification, options)


@click.command()
@click.option(
    '-d', '--dist-dir',
    type=click.Path(writable=True, file_okay=False),
    default='dist',
    help="Build eggs into <dir>.  Default is <cwd>/dist.",
    metavar='DIR',
    )
@click.argument(
    'wheels',
    nargs=-1,
    required=True,
    type=click.Path(exists=True, dir_okay=False),
    )
def main(dist_dir, wheels):
    """ Convert wheels to eggs.
    """

    logging.basicConfig(level=logging.WARNING, format="%(message)s")

    for wheel in wheels:
        EggWriter(wheel).build_egg(dist_dir)


if __name__ == '__main__':
    main()                      # pragma: NO COVER



# -*- coding: utf-8 -*-
from __future__ import absolute_import


def test_extension():
    from .test_ext import get_the_answer

    assert get_the_answer() == 42


def test_eager_resources():
    from .test_ext import read_the_answer

    assert read_the_answer() == 42



# -*- coding: utf-8 -*-
"""
"""
from __future__ import absolute_import

import re
from subprocess import call, check_call
import sys

import py
import pytest


@pytest.fixture(scope='session')
def session_tmpdir(request):
    tmpdir_handler = getattr(request.config, '_tmpdirhandler', None)
    if tmpdir_handler:
        # Create session tmpdir within pytest's session tmpdir
        return tmpdir_handler.mktemp('session', numbered=False)
    else:                       # pragma: NO COVER
        return py.path.local.make_numbered_dir('humpty-')


@pytest.fixture(scope='session')
def packages(session_tmpdir, request):
    return PackageManager(session_tmpdir, request)


class PackageManager(object):
    def __init__(self, tmpdir, request):
        self.tmpdir = tmpdir
        self.wheelhouse = tmpdir.join('wheelhouse')
        self.distdir = tmpdir.join('dist')
        self.wheels = {}
        self.eggs = {}
        self.venvs = {}
        self.saved_modes = []
        # restore modes so that pytest can delete the tmpdir
        request.addfinalizer(self._restore_modes)

    def _restore_modes(self):
        while self.saved_modes:
            path, mode = self.saved_modes.pop()
            path.chmod(mode)

    def get_wheel(self, dist_name, python_tag=None):
        key = dist_name, python_tag
        wheel = self.wheels.get(key)
        if wheel is None or True:
            tmpdir = py.path.local.make_numbered_dir('tmp', self.tmpdir)
            wheel = build_wheel(dist_name, self.wheelhouse, tmpdir,
                                python_tag=python_tag)
            self.wheels[key] = wheel
        return wheel

    def get_egg(self, dist_name):
        egg = self.eggs.get(dist_name)
        if egg is None:
            wheel = self.get_wheel(dist_name)
            egg = build_egg(wheel, self.distdir)
            self.eggs[dist_name] = egg
        return egg

    def require_eggs(self, *dists):
        for dist in dists:
            self.get_egg(dist)

    def get_venv(self, *dists, **kwargs):
        dists = frozenset(dists)
        unzip = kwargs.get('unzip', False)
        venv = self.venvs.get(dists)
        if venv is None:
            name = '-'.join(sorted(re.sub(r'\W', '_', dist) for dist in dists))
            if unzip:
                name += '-unzipped'
            vdir = self.tmpdir.join("venv_%s" % name)
            venv = Virtualenv(vdir, self.distdir, install=dists, unzip=unzip)
            self.venvs[dists] = venv

            # Make installed eggs read-only
            for p in vdir.visit(fil="*.egg"):
                if p.isdir():
                    self.saved_modes.append((p, p.stat().mode))
                    p.chmod(0o500)

        return venv


def build_wheel(dist_name, wheelhouse, tmpdir, python_tag=None):
    srcdir = py.path.local(__file__).dirpath(dist_name)
    setup_py = [sys.executable, 'setup.py']

    # Put all the build artifacts in our own private tmpdirs
    # so that simultaneous tox runs don't overwite each others builds
    ourtmp = tmpdir.ensure_dir(dist_name)
    setup_py.extend(['egg_info',
                     '--egg-base', str(ourtmp)])
    setup_py.extend(['build',
                     '--build-base', str(ourtmp.join('build')),
                     '--build-temp', str(ourtmp.join('btmp'))])

    bdist_wheel = ['bdist_wheel', '--dist-dir', str(ourtmp)]
    if python_tag is not None:
        bdist_wheel.extend(['--python-tag', python_tag])

    print("==== Building wheel for %s ====" % dist_name)
    with srcdir.as_cwd():
        check_call(setup_py + bdist_wheel)

    new_wheels = ourtmp.listdir(lambda f: f.isfile()
                                and f.fnmatch("%s-*.whl" % dist_name))
    assert len(new_wheels) == 1, "can't find newly created wheel"
    new_wheel = new_wheels[0]

    wheel = wheelhouse.ensure(dir=True).join(new_wheel.basename)
    new_wheel.move(wheel)
    return wheel


def build_egg(wheel, egg_dir):
    from humpty import EggWriter

    print("==== Building egg from %s ====" % wheel)
    egg = EggWriter(str(wheel)).build_egg(str(egg_dir))
    return py.path.local(egg)


class Virtualenv(object):
    def __init__(self, path, find_links=None, install=None, unzip=False):
        self.path = py.path.local(path)
        self.environ = {'PATH': str(self.path.join('bin'))}
        print("==== Creating virtualenv at %s ====" % path)
        check_call([sys.executable, '-m', 'virtualenv',
                    '--no-site', str(path)])

        if install:
            cmd = ['easy_install', '--index-url', 'file:///dev/null']
            if find_links:
                cmd.extend(['--find-links', str(find_links)])
            if unzip:
                cmd.append('--always-unzip')
            cmd.extend(install)
            self.check_call(cmd)

    def call(self, cmd, **kwargs):
        kwargs['env'] = self.environ
        return call(cmd, **kwargs)

    def check_call(self, cmd, **kwargs):
        kwargs['env'] = self.environ
        check_call(cmd, **kwargs)

    def run(self, prog, **kwargs):
        return self.call(['python', '-c', prog], **kwargs)

    def check_run(self, prog, **kwargs):
        return self.check_call(['python', '-c', prog], **kwargs)



# -*- coding: utf-8 -*-
"""
"""
from __future__ import absolute_import

from contextlib import contextmanager
import imp
import posixpath
from zipfile import ZipFile

from click.testing import CliRunner
import pkginfo
import pytest
from six import PY3


def test_pyfile_compiled(packages, tmpdir):
    packages.require_eggs('dist1')
    unzip = False
    if PY3:
        # Python >= 3.2 doesn't seem to run .pyc files from PEP 3147
        # (__pycache__) repository directories.
        unzip = True
    venv = packages.get_venv('dist1', unzip=unzip)
    assert venv.run("__import__('dist1').test_is_compiled()") == 0


@pytest.fixture
def dist1_metadata(packages):
    egg = packages.get_egg('dist1')
    return pkginfo.BDist(str(egg))


def test_summary(dist1_metadata):
    assert dist1_metadata.summary == "A dummy distribution"


def test_description(dist1_metadata):
    assert dist1_metadata.description.rstrip() \
        == u"Long description.\n\nGruß."


def test_script_wrapper(packages):
    packages.require_eggs('dist1')
    venv = packages.get_venv('dist1')
    assert venv.call(['dist1_wrapper']) == 42


def test_old_style_script(packages):
    packages.require_eggs('dist1')
    venv = packages.get_venv('dist1')
    assert venv.call(['dist1_script']) == 42


def test_namespace_package(packages):
    packages.require_eggs('dist1', 'dist2')
    venv = packages.get_venv('dist2')
    prog = (
        'import sys\n'
        'from dist2.plugins.builtin import the_answer\n'
        'sys.exit(the_answer)\n'
        )
    assert venv.run(prog) == 42


def test_namespace_stubs_in_egg(packages):
    dist2_egg = packages.get_egg('dist2')
    dist2_stubs = with_byte_compiled(['dist2/__init__.py',
                                      'dist2/plugins/__init__.py'])
    with fileobj(ZipFile(str(dist2_egg))) as zf:
        files_in_egg = dist2_stubs.intersection(zf.namelist())

    # Make sure we generated the stubs (or not, depending on python
    # version)
    stubs_in_egg = files_in_egg.intersection(dist2_stubs)
    assert stubs_in_egg == dist2_stubs

    # Make sure we didn't copy the .pth file that the wheel installer
    # creates for the namespaces
    assert not any(fn.lower().endswith('.pth')
                   for fn in files_in_egg)


def test_extension(packages):
    packages.require_eggs('extension_dist')
    venv = packages.get_venv('extension_dist')
    assert venv.run("__import__('extension_dist').test_extension()") == 0


def test_eager_resources(packages):
    packages.require_eggs('extension_dist')
    venv = packages.get_venv('extension_dist')
    assert venv.run("__import__('extension_dist').test_eager_resources()") == 0


def test_extras(packages):
    packages.require_eggs('dist1', 'extension_dist')
    venv = packages.get_venv('dist1[extras]')
    assert venv.run("__import__('dist1').test_extras()") == 0


def test_no_extras(packages):
    packages.require_eggs('dist1', 'extension_dist')
    venv = packages.get_venv('dist1')
    assert venv.run("__import__('dist1').test_no_extras()") == 0


def test_main(packages, tmpdir):
    from humpty import main

    wheel = packages.get_wheel('dist1')

    runner = CliRunner()
    result = runner.invoke(main, ['-d', str(tmpdir), str(wheel)])
    assert result.exit_code == 0

    eggs = list(tmpdir.listdir(fil="*.egg"))
    assert len(eggs) == 1
    egg = eggs[0]
    assert egg.isfile()
    assert egg.fnmatch("dist1-*")


@contextmanager
def fileobj(fp):
    try:
        yield fp
    finally:
        fp.close()


def with_byte_compiled(paths):
    """ Augment PATHS with paths of byte-compiled files.
    """
    get_tag = getattr(imp, 'get_tag', None)
    compiled = set()
    for path in paths:
        head, tail = posixpath.split(path)
        root, ext = posixpath.splitext(tail)
        if ext == '.py':
            if get_tag:
                root = '%s.%s' % (root, get_tag())
                head = posixpath.join(head, '__pycache__')
            compiled.add(posixpath.join(head, root + '.pyc'))
    return compiled.union(paths)



# -*- coding: utf-8 -*-
""" Unit tests.
"""
from __future__ import absolute_import

import imp
import os
import posixpath
import sys
from zipfile import ZipFile

from pkg_resources import parse_version, require
import pytest
from six import int2byte, unichr, StringIO

try:
    import sysconfig
except ImportError:             # python < 2.7
    # FIXME: incorrect on non-posix systems
    EXT_SUFFIX = '.so'
else:
    EXT_SUFFIX = (sysconfig.get_config_var('EXT_SUFFIX')
                  or sysconfig.get_config_var('SO'))


class test_cache_from_source():
    from humpty import cache_from_source
    if sys.version_info >= (3, 2):
        tag = imp.get_tag()
        assert cache_from_source('foo.py') == os.path.join('__pycache__',
                                                           'foo.%s.pyc' % tag)
    else:
        assert cache_from_source('foo.py') == 'foo.pyc'


class Test_file_cm(object):
    def make_one(self, fp):
        from humpty import file_cm
        return file_cm(fp)

    def test_closes_file(self):
        with self.make_one(StringIO()) as fp:
            assert not fp.closed
        assert fp.closed

    def test_proxy(self):
        fp = StringIO()
        proxy = self.make_one(fp)
        proxy.write('x')
        assert fp.getvalue() == 'x'


def test_unsplit_sections():
    from humpty import unsplit_sections
    content = unsplit_sections([
        (None, ['a', 'b']),
        ('foo', ['c']),
        ])
    assert content == (b"a\n"
                       b"b\n"
                       b"[foo]\n"
                       b"c\n")


def test_join_lines():
    from humpty import join_lines
    assert join_lines(['a', 'b']) == b'a\nb\n'
    assert join_lines([u'ø']) == b'\xc3\xb8\n'


def test_bytes():
    from humpty import bytes_
    all_bytes = b''.join(map(int2byte, range(256)))
    chars256 = ''.join(map(unichr, range(256)))
    assert bytes_(all_bytes) == all_bytes
    assert bytes_(chars256) == all_bytes


class DummyWheelMetadata(object):
    exports = {}
    namespaces = []
    extras = []
    run_requires = []
    index_data = {}

    _legacy = {}

    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)

    def todict(self):
        return self.index_data


class EggInfoTestBase(object):
    @pytest.fixture
    def index_data(self):
        return {}

    @pytest.fixture
    def wheel_metadata(self, index_data):
        return DummyWheelMetadata(index_data=index_data)

    @pytest.fixture
    def installed_files(self):
        return set([])

    @pytest.fixture
    def metadata_files(self):
        return {
            'METADATA': b'',
            }

    def test_files(self, egg_info, wheel_metadata):
        egg_info.zip_safe = True
        pkg_info = '\n'.join(egg_info.pkg_info) + '\n'
        assert dict(egg_info.files()) == {
            'PKG-INFO': pkg_info.encode('utf-8'),
            'zip-safe': b"",
            }

    def test_files_zip_safe(self, egg_info):
        egg_info.zip_safe = True
        files = dict(egg_info.files())
        assert files['zip-safe'] == b''
        assert 'not-zip-safe' not in files

    def test_files_not_zip_safe(self, egg_info):
        egg_info.zip_safe = False
        files = dict(egg_info.files())
        assert files['not-zip-safe'] == b''
        assert 'zip-safe' not in files

    def test_pkg_info(self, egg_info, index_data):
        index_data.update({
            'name': 'dummy',
            'version': '3.2.1',
            'author': 'Joe',
            'author_email': 'joe@example.com',
            'description': 'Line 1\nLine 2',
            })
        assert list(egg_info.pkg_info) == [
            'Metadata-Version: 1.1',
            'Name: dummy',
            'Version: 3.2.1',
            'Summary: UNKNOWN',
            'Home-page: UNKNOWN',
            'Author: Joe',
            'Author-email: joe@example.com',
            'License: UNKNOWN',
            'Description: Line 1',
            '        Line 2',
            'Platform: UNKNOWN',
            ]

    def test_pkg_info_download_url(self, egg_info, index_data):
        index_data['download_url'] = 'http://example.org'
        assert 'Download-URL: http://example.org' in egg_info.pkg_info

    def test_pkg_info_description(self, egg_info, index_data):
        index_data['description'] = "Line 1\nLine 2"
        pkg_info = list(egg_info.pkg_info)
        i = pkg_info.index('Description: Line 1')
        assert pkg_info[i+1] == '        Line 2'

    def test_pkg_info_keywords(self, egg_info, index_data):
        index_data['keywords'] = ['a', 'b']
        assert 'Keywords: a,b' in egg_info.pkg_info

    def test_pkg_info_platform(self, egg_info, index_data):
        index_data['platform'] = ['foo', 'bar']
        assert 'Platform: foo' in egg_info.pkg_info
        assert 'Platform: bar' in egg_info.pkg_info
        assert 'Platform: UNKNOWN' not in egg_info.pkg_info

    def test_pkg_info_classifiers(self, egg_info, index_data):
        index_data['classifiers'] = ['foo', 'bar']
        assert 'Classifier: foo' in egg_info.pkg_info
        assert 'Classifier: bar' in egg_info.pkg_info
        assert 'Classifier: UNKNOWN' not in egg_info.pkg_info


class TestEggInfo_Legacy(EggInfoTestBase):
    @pytest.fixture
    def egg_info(self, wheel_metadata, installed_files, metadata_files):
        from humpty import EggInfo_Legacy
        return EggInfo_Legacy(wheel_metadata, installed_files, metadata_files)

    def test_files_requires_txt(self, egg_info, wheel_metadata):
        wheel_metadata.run_requires = [u'req']
        files = dict(egg_info.files())
        assert files['requires.txt'] == b'req\n'

    def test_files_namespace_packages_txt(self, egg_info, metadata_files):
        metadata_files['namespace_packages.txt'] = b'foo\nfoo.bar\n'
        files = dict(egg_info.files())
        assert files['namespace_packages.txt'] == b'foo\nfoo.bar\n'

    def test_get_index_data(self, egg_info):
        assert 'description' in egg_info._get_index_data()

    def test_get_index_data_existing_description(self, egg_info, index_data):
        index_data['description'] = 'Test'
        assert egg_info._get_index_data()['description'] == 'Test'

    def test_get_description(self, egg_info, metadata_files):
        metadata_files['METADATA'] = (
            u"Metadata-Version: 1.1\n"
            u"Name: something\n"
            u"\n"
            u"Fü\n"
            u"Line 2\n"
            ).encode('utf-8')
        assert egg_info._get_description() == u"Fü\nLine 2\n"

    def test_entry_points(self, egg_info, metadata_files):
        metadata_files['entry_points.txt'] = (
            b"# testing\n"
            b"[console_scripts]\n"
            b"mycmd = foo:main\n"
            )
        assert egg_info.entry_points == [
            ('console_scripts', ["mycmd = foo:main"]),
            ]

    def test_entry_points_empty(self, egg_info):
        assert egg_info.entry_points == []

    def test_namespace_packages(self, egg_info, metadata_files):
        metadata_files['namespace_packages.txt'] = b'foo\n#x\nfoo.bar\n'
        assert egg_info.namespace_packages == ['foo', 'foo.bar']

    def test_top_level(self, egg_info, metadata_files):
        metadata_files['top_level.txt'] = b'foo\n'
        assert egg_info.top_level == ['foo']

    def test_top_level_fallback(self, egg_info, installed_files):
        installed_files.add('bar/baz.py')
        assert egg_info.top_level == ['bar']

    def test_eager_resources(self, egg_info, metadata_files):
        metadata_files['eager_resources.txt'] = b'foo/data.txt\n'
        assert egg_info.eager_resources == ['foo/data.txt']

    def test_requires_rfc822(self, egg_info, wheel_metadata):
        wheel_metadata.run_requires = [
            u"foo",
            u"bar ; extra == 'addon'",
            u'baz ; extra == "addon"',
            u"bozo ; extra == 'ignored' and sys_platform == 'goober'",
            ]
        wheel_metadata.extras = ['addon', 'ignored']
        assert egg_info.requires == [
            (None, ['foo']),
            ('addon', ['bar', 'baz']),
            ]

    def test_requires_json(self, egg_info, wheel_metadata):
        wheel_metadata._legacy = None
        wheel_metadata.run_requires = [
            {
                'requires': ["foo"],
                },
            {
                'requires': ["bar", "baz"],
                'extra': 'addon',
                },
            {
                'requires': ["bozo"],
                'extra': 'ignored',
                'environment': "sys_platform == 'goober'"
                },
            ]
        assert egg_info.requires == [
            (None, ['foo']),
            ('addon', ['bar', 'baz']),
            ]


class TestEggInfo(EggInfoTestBase):
    @pytest.fixture
    def egg_info(self, wheel_metadata, installed_files, metadata_files):
        from humpty import EggInfo
        return EggInfo(wheel_metadata, installed_files, metadata_files)

    def test_files_requires_txt(self, egg_info, wheel_metadata):
        wheel_metadata.run_requires = [{'requires': ['req']}]
        files = dict(egg_info.files())
        assert files['requires.txt'] == b'req\n'

    def test_files_namespace_packages_txt(self, egg_info, wheel_metadata):
        wheel_metadata.namespaces = ['foo', 'foo.bar']
        files = dict(egg_info.files())
        assert files['namespace_packages.txt'] == b'foo\nfoo.bar\n'

    def test_top_level(self, egg_info, installed_files):
        installed_files.update([
            'a/module.py',
            'mod' + EXT_SUFFIX,
            'dir/data.txt',
            'a/b/module.py',
            ])
        assert egg_info.top_level == ['a', 'mod']

    def test_native_libs(self, egg_info, installed_files):
        installed_files.update([
            'a/module.py',
            'a/ext.so',
            'foo.dll',
            ])
        assert egg_info.native_libs == ['a/ext.so', 'foo.dll']

    def test_entry_points(self, egg_info, wheel_metadata):
        wheel_metadata.exports = {
            'console_scripts': {
                'mycmd': 'foo:main',
                },
            }
        assert egg_info.entry_points == [
            ('console_scripts', ["mycmd = foo:main"]),
            ]

    def test_namespace_packages(self, egg_info, wheel_metadata):
        wheel_metadata.namespaces = ['foo', 'foo.bar']
        assert egg_info.namespace_packages == ['foo', 'foo.bar']

    def test_eager_resources(self, egg_info):
        # coverage
        assert egg_info.eager_resources == []

    def test_requires(self, egg_info, wheel_metadata):
        wheel_metadata.run_requires = [
            {
                'requires': ['foo'],
                },
            {
                'extra': 'addon',
                'requires': ['bar', 'baz'],
                },
            ]
        wheel_metadata.extras = ['addon']
        assert egg_info.requires == [
            (None, ['foo']),
            ('addon', ['bar', 'baz']),
            ]


class DummyWheel(object):
    mountable = True

    def __init__(self, **kwargs):
        self.__dict__.update(**kwargs)

    def is_mountable(self):
        return self.mountable


@pytest.fixture
def wheel_files():
    return {
        'mod.py': b'# mod.py\n',
        'pkg/mod2.py': b"# mod2.py\n",
        'distname-1.0.data/purelib/mod3.py': b"# mod3.py\n",
        'distname-1.0.data/platlib/ext' + EXT_SUFFIX: b"",
        'distname-1.0.data/other/junk.txt': b"Junk\n",
        'distname-1.0.dist-info/WHEEL': b"Wheel-Version: 1.0\n",
        'distname-1.0.dist-info/METADATA': b"Metadata-Version: 2.0\n",
        }


@pytest.fixture
def wheel_version():
    return '1.0'


@pytest.fixture
def wheel_is_mountable():
    return True


@pytest.fixture
def dummy_wheel(tmpdir, wheel_files, wheel_version, wheel_is_mountable):
    wheel_file = tmpdir.join('dummy.whl')
    zf = ZipFile(str(wheel_file), 'w')
    for path, content in wheel_files.items():
        zf.writestr(path, content)
    zf.close()
    return DummyWheel(
        name='distname',
        version='1.0',
        dirname=str(tmpdir),
        filename='dummy.whl',
        mountable=wheel_is_mountable,
        info={'Wheel-Version': wheel_version},
        metadata=DummyWheelMetadata())


def test_list_installed_files(dummy_wheel):
    from humpty import list_installed_files
    assert list_installed_files(dummy_wheel) == set([
        'mod.py',
        'pkg/mod2.py',
        'mod3.py',
        'ext' + EXT_SUFFIX,
        ])


def test_read_metadata_files(dummy_wheel):
    from humpty import read_metadata_files
    assert read_metadata_files(dummy_wheel) == {
        'WHEEL': b"Wheel-Version: 1.0\n",
        'METADATA': b"Metadata-Version: 2.0\n",
        }


@pytest.mark.parametrize('wheel_is_mountable', [True, False])
def test_is_zip_safe(dummy_wheel, wheel_is_mountable):
    from humpty import is_zip_safe
    assert is_zip_safe(dummy_wheel) is bool(wheel_is_mountable)


def test_get_wheel_version(dummy_wheel):
    from humpty import get_wheel_version
    assert get_wheel_version(dummy_wheel) == (1, 0)


@pytest.mark.parametrize('wheel_version, egg_info_class', [
    ('1.0', 'EggInfo_Legacy'),
    ('1.1', 'EggInfo'),
    ])
def test_egg_metadata(dummy_wheel, egg_info_class):
    from humpty import egg_metadata
    egg_info = egg_metadata(dummy_wheel)
    assert type(egg_info).__name__ == egg_info_class


class DummyEggInfo(object):
    namespace_packages = []
    native_libs = []
    zip_safe = True

    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)


def with_byte_compiled(paths):
    """ Augment PATHS with paths of byte-compiled files.
    """
    get_tag = getattr(imp, 'get_tag', None)
    compiled = set()
    for path in paths:
        head, tail = posixpath.split(path)
        root, ext = posixpath.splitext(tail)
        if ext == '.py':
            if get_tag:
                root = '%s.%s' % (root, get_tag())
                head = posixpath.join(head, '__pycache__')
            compiled.add(posixpath.join(head, root + '.pyc'))
    return compiled.union(paths)


class TestStubLoaders(object):
    @pytest.fixture
    def egg_info(self):
        return DummyEggInfo()

    @pytest.fixture
    def egg_name(self):
        return 'dummy.egg'

    @pytest.fixture
    def stub_loaders(self, egg_info, egg_name):
        from humpty import StubLoaders
        return StubLoaders(egg_info, egg_name)

    def test_namespace_stubs_for_py2(self, stub_loaders, egg_info):
        egg_info.namespace_packages = ['foo']
        stubs = dict(stub_loaders)
        assert set(stubs) == with_byte_compiled(['foo/__init__.py'])

    def test_extension_stubs(self, stub_loaders, egg_info):
        egg_info.native_libs = ['ext' + EXT_SUFFIX]
        stubs = dict(stub_loaders)
        assert set(stubs) == with_byte_compiled(['ext.py'])

    def test_no_extension_stubs_if_zip_unsafe(self, stub_loaders, egg_info,
                                              monkeypatch):
        egg_info.native_libs = ['ext' + EXT_SUFFIX]
        egg_info.zip_safe = False
        stubs = dict(stub_loaders)
        assert set(stubs) == set([])

    def test_namespace_stubs(self, stub_loaders, egg_info):
        egg_info.namespace_packages = ['foo', 'foo.bar']
        stubs = dict(stub_loaders.namespace_stubs())
        assert set(stubs) == set(['foo/__init__.py', 'foo/bar/__init__.py'])

    def test_extension_stub_loaders(self, stub_loaders, egg_info):
        egg_info.native_libs = ['ext' + EXT_SUFFIX]
        loaders = dict(stub_loaders.extension_stub_loaders())
        assert set(loaders) == set(['ext.py'])


def test_warner(caplog):
    # coverage
    from humpty import warner
    warner((1, 2), (1, 0))
    assert "Wheel version mismatch" in caplog.text


class TestScriptCopyer(object):
    @pytest.fixture
    def srcdir(self, tmpdir):
        return tmpdir.join('src').ensure(dir=True)

    @pytest.fixture
    def dstdir(self, tmpdir):
        return tmpdir.join('dst').ensure(dir=True)

    @pytest.fixture
    def copyer(self, srcdir, dstdir):
        from humpty import ScriptCopyer
        return ScriptCopyer(str(srcdir), str(dstdir))

    def test_copies_script(self, copyer, srcdir, dstdir):
        srcdir.join('tester').write(
            "#!python\n"
            "print('Hello')\n"
            )
        dstpath = dstdir.join('tester')

        scripts = copyer.make('tester')

        assert scripts == [str(dstpath)]
        result = dstdir.join('tester').open().read()

        # Remove the blank line that some versions of distlib add
        # after the hashbang
        result_lines = list(filter(None, result.splitlines()))

        # Distlib==0.2.6 adds the new hashbang as the first line, but
        # leaves the original in place in the second line.
        if result_lines[1].startswith('#'):
            result_lines.pop(1)

        assert result_lines == [
            "#!%s" % sys.executable,
            "print('Hello')",
            ]

    def test_skips_wrapper(self, copyer, dstdir):
        scripts = copyer.make('tester = foo:main')
        assert scripts == []
        assert len(dstdir.listdir()) == 0


# Wheel==0.23 is the first version which supports the --python-tag
# argument to bdist_wheel.
skip_if_python_tag_unsupported = pytest.mark.skipif(
    require('wheel')[0].parsed_version < parse_version('0.23'),
    reason="Installed version of wheel does not support --python-tag")


class TestEggWriter(object):
    def make_one(self, wheel_file):
        from humpty import EggWriter
        return EggWriter(str(wheel_file))

    @skip_if_python_tag_unsupported
    def test_warns_on_incompatibility(self, packages, caplog):
        wheel_file = packages.get_wheel('dist1', python_tag='bogus12')
        self.make_one(wheel_file)
        assert len(caplog.records) == 1
        assert 'not compatible' in caplog.records[0].getMessage()

    def test_no_warning_if_compatible(self, packages, caplog):
        wheel_file = packages.get_wheel('dist1')
        self.make_one(wheel_file)
        assert len(caplog.records) == 0



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
setup(
    name='dist2',
    version='0.2',
    description="Another dummy distribution",
    long_description="Long description.",
    classifiers=[
        "Topic :: Software Development :: Testing",
        ],
    author='Jeff Dairiki',
    author_email='dairiki@dairiki.org',
    keywords='dummy testing',
    packages=find_packages(),
    namespace_packages=['dist2', 'dist2.plugins'],
    install_requires=['dist1'],
    )



# -*- coding: utf-8 -*-
import sys


def main():
    sys.exit(42)


def test_is_compiled():
    global __cached__, __file__
    try:
        source = __cached__ or __file__
    except NameError:
        source = __file__
    assert source.endswith('.pyc')


def test_extras():
    from extension_dist.test_ext import get_the_answer

    assert get_the_answer() == 42


def test_no_extras():
    try:
        import extension_dist   # noqa
    except ImportError:
        pass
    else:
        assert False, "extra was insatlled when it shouldn't have been"



# -*- coding: utf-8 -*-
from setuptools import setup, Extension
setup(
    name='extension_dist',
    version='0.1',
    description="A dummy distribution",
    long_description="A distribution with an extension module.",
    classifiers=[
        "Topic :: Software Development :: Testing",
        ],
    author='Jeff Dairiki',
    author_email='dairiki@dairiki.org',
    keywords='dummy testing',
    packages=['extension_dist'],
    ext_modules=[
        Extension('extension_dist.test_ext', ['test_ext.c']),
        ],
    eager_resources=[
        'extension_dist/answer.dat',
        ],
    package_data={
        'extension_dist': ['*.dat'],
        },
    )



# -*- coding: utf-8 -*-
import os
import sys

from setuptools import setup
from setuptools.command.test import test as TestCommand

VERSION = '0.2.1'

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()
CHANGES = open(os.path.join(here, 'CHANGES.rst')).read()

requires = [
    'click',
    'distlib',
    'setuptools',
    'six',
    ]

tests_require = [
    'pytest',
    'pytest-catchlog',
    'pkginfo',
    'virtualenv',
    ]


class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', "Arguments to pass to py.test")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


cmdclass = {'test': PyTest}

setup(
    name='humpty',
    version=VERSION,
    description="A tool to convert python wheels to eggs",
    long_description=README + '\n\n' + CHANGES,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Topic :: System :: Archiving :: Packaging",
        "Topic :: System :: Software Distribution",
        ],
    author='Jeff Dairiki',
    author_email='dairiki@dairiki.org',
    url='https://github.com/dairiki/humpty',
    keywords='python packaging wheel whl egg',

    py_modules=['humpty'],
    install_requires=requires,

    include_package_data=True,
    zip_safe=True,

    entry_points={
        'console_scripts': [
            'humpty = humpty:main',
            ],
        },

    tests_require=tests_require,
    cmdclass=cmdclass,
    extras_require={
        "testing": tests_require,
        },
    )



# -*- coding: utf-8 -*-
from setuptools import setup
setup(
    name='dist1',
    version='0.1',
    description="A dummy distribution",
    long_description=u"Long description.\n\nGruß.\n",
    classifiers=[
        "Topic :: Software Development :: Testing",
        ],
    author='Jeff Dairiki',
    author_email='dairiki@dairiki.org',
    keywords='dummy testing',
    py_modules=['dist1'],
    install_requires=[],
    scripts=[
        'dist1_script',
        ],
    entry_points={
        'console_scripts': [
            'dist1_wrapper = dist1:main',
            ],
        },
    extras_require={
        'extras': ['extension_dist'],
        },
    )
